function [model_list  batch_run] = CBM_Exp_Load(model_list, batch_run)
    model_list = OptBatchLoad(model_list, batch_run, 2);
    
    for i=1:length(model_list)
        model_list{i} = ComputeActivityScores(model_list{i});
    end
end%function

function [model] = ComputeActivityScores(model)
    Private_DefineParameters;

    v = find(model.batch_type==BATCH_FORWARD);    u = model.batch_ind(v);
    model.reactions_forward_score(u) = model.result_opt(v);
    model.reactions_forward_score(u(model.result_status(v)==0)) = 0;%if status is infeasible then scr =0
    model.reactions_forward_mat = sparse(model.rxn_num, model.var_num);
    model.reactions_forward_mat(u, :) = model.result_vector(v, :);
    model.reactions_forward_mat = ProcessPredVec(model, model.reactions_forward_mat);

    v = find(model.batch_type==BATCH_BACKWARD);     u = model.batch_ind(v);
    model.reactions_backward_score(u) = model.result_opt(v);
    model.reactions_backward_score(u(model.result_status(v)==0)) = 0;
    model.reactions_backward_mat = sparse(model.rxn_num, model.var_num);
    model.reactions_backward_mat(u, :) = model.result_vector(v, :);
    model.reactions_backward_mat = ProcessPredVec(model, model.reactions_backward_mat);
    
    v = find(model.batch_type==BATCH_INACTIVE);    u = model.batch_ind(v);
    model.reactions_inactive_score(u) = model.result_opt(v);
    model.reactions_inactive_score(u(model.result_status(v)==0)) = 0;
    model.reactions_inactive_mat = sparse(model.rxn_num, model.var_num);
    model.reactions_inactive_mat(u, :) = model.result_vector(v, :);
    model.reactions_inactive_mat = ProcessPredVec(model, model.reactions_inactive_mat);

    %v = find(model.batch_type==BATCH_GENE_KO);    u = model.batch_ind(v);
    %model.gene_ko_score(u) = model.result_opt(v);
    %model.gene_ko_score(u(model.result_status(v)==0)) = 0;
    % Handle single reaction genes
    %v = find(model.gene_ko_single_reaction);
    %model.gene_ko_score(v) = model.reactions_inactive_score(model.gene_ko_single_reaction(v));
    
    %%%%%%%%%%%%%%%
    
    % REACTION ACTIVITY SCORE
    model.reactions_activity_score = zeros(model.rxn_num, 1)+nan;
    v = max(model.reactions_forward_score, model.reactions_backward_score);
    u = find(v>-1 & model.reactions_inactive_score>-1); %-1 means that run was completed and a scr exists (even if scr=0)
    model.reactions_activity_score(u) = v(u) - model.reactions_inactive_score(u);

    model.reactions_mat = sparse(model.rxn_num, model.rxn_num);
    u = model.reactions_activity_score>0;
    model.reactions_mat(u, :) = model.reactions_inactive_mat(u, :);
    u = model.reactions_activity_score<=0 & model.reactions_forward_score > model.reactions_backward_score ;
    model.reactions_mat(u, :) = model.reactions_forward_mat(u, :);
    u = model.reactions_activity_score<=0 & model.reactions_forward_score < model.reactions_backward_score ;
    model.reactions_mat(u, :) = model.reactions_backward_mat(u, :);
    
    % REACTION DIRECTION SCORE
    model.reactions_direction_score = zeros(model.rxn_num, 1)+nan;
    %  v = max(model.reactions_forward_score, model.reactions_backward_score);
    u = find(model.reactions_activity_score>0 & ~isnan(model.reactions_forward_score) & ~isnan(model.reactions_backward_score) );
    model.reactions_direction_score(u) = model.reactions_forward_score(u) - model.reactions_backward_score(u);
    
    % GENE ACTIVITY SCORE
    %reaction_active = max(model.reactions_forward_score, model.reactions_backward_score);
    %reaction_active_mat = repmat(reaction_active, 1, model.gene_num);
    %gene_active = max( model.rxnGeneMat .* reaction_active_mat )'+0;
    %model.genes_activity_score = zeros(model.gene_num, 1)+nan;
    %u = find(gene_active>-1 & model.gene_ko_score>-1);
    %model.genes_activity_score(u) = gene_active(u) - model.gene_ko_score(u);

    %reaction_active_mat = repmat(model.reactions_activity_score, 1, model.gene_num);
    %m = model.rxnGeneMat;
    %m(m==0) = nan;
    %gene_active = max( m .* reaction_active_mat )'+0;    
    %model.genes_activity_score = zeros(model.gene_num, 1)+nan;    
    %u = find(gene_active>-1 & model.gene_ko_score>-1);
    %model.genes_activity_score(u) = gene_active(u) - model.gene_ko_score(u)    
    
    model.genes_activity_score = 0;
    for i=1:model.gene_num
        v = find(model.rxnGeneMat(:, i));
        u = max(model.reactions_activity_score(v));
        model.genes_activity_score(i,1) = u;
    end
end%function

    
function [mat2] = ProcessPredVec(model, mat)
    mat2 = sparse(size(mat,1), model.rxn_num);
    v = find(model.var_type == 1);
    u = model.var_ind(v);
    mat2(:, u) = mat(:, v);
    
    v = find(model.var_type == -1);
    u = model.var_ind(v);
    mat2(:, u) = mat2(:, u) + mat(:, v);

    v = find(model.var_type == 0); %what is var_type??????
    u = model.var_ind(v);
    mat2(:, u) = -mat(:, v);
end%function
