function [batch_run]= OptBatchRun(prob_list, save_vectors, batch_max_size, max_machine_num, batch_name,m_pwd)
    if (exist('win.txt','file'))
        win = 1;
    else
        win = 0;
    end
    
    if win==0
        pref = sprintf('%s/%s', m_pwd, batch_name);
        cmd = sprintf('rm -f %s/*', pref);    %fprintf('CMD: %s\n', cmd);
        %system(cmd);
    else
        pref = sprintf('%s\\%s', m_pwd, batch_name);
        cmd = sprintf('del /q %s\\*', pref);    %fprintf('CMD: %s\n', cmd);
        %system(cmd);
    end

    cmd = sprintf('mkdir %s', pref);    %fprintf('CMD: %s\n', cmd);
    system(cmd);
    batch_run.pref = pref;
    
    if (win==0 & batch_max_size > 0)
        fn = sprintf('%s/run.cmd', pref);
        f3 = fopen(fn, 'wt');
        fprintf(f3, 'condor_submit_dag -maxjobs %d simple.dag\n', max_machine_num);
        fprintf(f3, 'condor_reschedule\n');
        fclose(f3);
    end

    fn = sprintf('%s/simple.dag', pref);
    f2 = fopen(fn, 'wt');
    fn2 = sprintf('%s/jobs2submit.txt', pref);
    f22 = fopen(fn2, 'wt');
    job_count = 1;
    
    for i=1:length(prob_list) %print for each problem in the list all the input files for cplex_wrapper
        mat_f = sprintf('prob_%d_mat.txt', i);
        SaveMatrixSparse(sprintf('%s/%s', pref, mat_f), prob_list{i}.S);
        
        row_f = sprintf('prob_%d_row_bound.txt', i);
        SaveMatrixFull(sprintf('%s/%s', pref, row_f),[prob_list{i}.rowlb prob_list{i}.rowub]);
        
        col_f = sprintf('prob_%d_col_bound.txt', i);
        SaveMatrixFull(sprintf('%s/%s', pref, col_f),[prob_list{i}.lb prob_list{i}.ub]);

        opt_f = sprintf('prob_%d_opt.txt', i);
        SaveMatrixSparse(sprintf('%s/%s', pref, opt_f), prob_list{i}.c);

        int_f = sprintf('prob_%d_int.txt', i);
        SaveMatrixFull(sprintf('%s/%s', pref, int_f), prob_list{i}.int_vars);

%         for x=1:size(prob_list{i}.batch_lb,2)
%             t = prob_list{i}.batch_lb(:, x) ~= prob_list{i}.lb;
%             prob_list{i}.batch_lb(t, x) = prob_list{i}.batch_lb(t, x) + 1234;
% 
%             t = prob_list{i}.batch_ub(:, x) ~= prob_list{i}.ub;
%             prob_list{i}.batch_ub(t, x) = prob_list{i}.batch_ub(t, x) + 1234;
%         end
        
        if (batch_max_size > 0)
            batch_f = sprintf('prob_%d_batch_lb.txt', i);
            SaveMatrixSparse(sprintf('%s/%s', pref, batch_f), prob_list{i}.batch_lb);

            batch_f = sprintf('prob_%d_batch_ub.txt', i);
            SaveMatrixSparse(sprintf('%s/%s', pref, batch_f), prob_list{i}.batch_ub);
        
            n = size(prob_list{i}.batch_lb,2); %number of batch problems
            
            col_ind = 1;
            c=1;
            while (col_ind <= n)
                
                fn = sprintf('%s/run_%d_%d', pref, i, col_ind);
                f1 = fopen(fn, 'wt');
                fprintf(f1, 'executable = %s/cplex_wrapper\n', m_pwd);
                fprintf(f1, 'log = %s/prob_%d_%d.log\n',pref, i, col_ind);
                fprintf(f1, 'error = %s/prob_%d_%d.error\n',pref, i, col_ind);
                fprintf(f1, 'output =%s/prob_%d_%d.output\n',pref, i, col_ind);
                fprintf(f1, 'requirements = substr(machine,0,6) == "safraf" && arch == "X86_64"\n');
                fprintf(f1, 'universe = vanilla\n');
                fprintf(f1, 'getenv = True\n');
                
                s = min(batch_max_size, n-col_ind+1); %number of batch problems for current run( does not exceed max_size) 

                fprintf(f1, 'arguments = "%s/ %d %d %d %d %d %d %d %d"\n', pref, i, ...
                                size(prob_list{i}.S,1), size(prob_list{i}.S,2), save_vectors, 1, col_ind, s, 0);%last paramter is skip mode

                fprintf(f1, 'queue 1\n');
                fclose(f1);
                
                fprintf(f2, 'Job J%d %s/run_%d_%d\n', job_count, pref, i, col_ind);
                fprintf(f22, '%s/run_%d_%d\n',pref, i, col_ind);
               
                batch_run.data{i}(c,1) = col_ind;%first index out of the total batch,in this run
                batch_run.data{i}(c,2) = s;%number of problems out of the batch that were solved
                c=c+1;

                col_ind = col_ind + s;
                job_count = job_count + 1;
            end
        else %do not run in batch mode, 
            fprintf(f2, '%s/cplex_wrapper %s/ %d %d %d %d %d\n', m_pwd, pref, i, ...
                            size(prob_list{i}.S,1), size(prob_list{i}.S,2), save_vectors, 0);
            try
                if (win ==0)%run simulation now
                    cmd = sprintf('tcsh %s', fn);    %fprintf('CMD: %s\n', cmd);
                    system(cmd);
                end
            catch
                fprintf('ERROR: Failed running optimization\n');
            end
            batch_run.data{1}(1, 1) = 1;
            batch_run.data{1}(1, 2) = 1;
            [batch_run]= OptBatchLoad(prob_list, batch_run, 1);
        end
    end
    
    fclose(f2);
    fclose(f22);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = SaveMatrixSparse(file_name,matrix1)

 out = fopen(file_name, 'wt');
    [a,b,c] = find(matrix1);
    for i=1:length(c)
        fprintf(out, '%d\t%d\t%d\n', a(i), b(i), c(i));
        %str = sprintf('i = %d',i);
        %disp(str);
    end
fclose(out);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = SaveMatrixFull(file_name, matrix1)

num_of_rows = size(matrix1,1);
num_of_cols = size(matrix1,2);
out = fopen(file_name, 'wt');
    for y = 1:size(matrix1,1)
        for x = 1:size(matrix1,2)
           fprintf(out, '%d ',  matrix1(y,x));
        end
        fprintf(out, '\n');
    end
    fclose(out);

