function [milp] = ProcessModelExp(model, exp_data, reaction_data, model_name)

%WARNING: NEW CONSTARINTS ON ACT/INACT USE flux_bound instead of inf_const

%Description: recieves a model a trinary vector  of expresion data and a trinary vector 
%of reaction data and creates a mixed integer linear problem that
%integrates the expression with the model.
%The trinary vector holds -1,0,1 for low, moderate and high respectivaly.

    Private_DefineParameters;
    inf_const = 999 ;

    % REACTION AND EXP CONSTRAINTS
    if isempty(reaction_data)
        reaction_data = zeros(length(model.lb),1);
    end

    if ~isempty(exp_data)
        for i = 1:length(model.rules)
            reaction_data(i) = Private_EvalExpEquation(model.rules{i}, exp_data);
        end
    end
    fprintf('Model gene expression: up=%d, Down=%d, Other=%d\n', sum(exp_data==1), sum(exp_data==-1), sum(exp_data==0));
    fprintf('Model reaction expression: up=%d, Down=%d, Other=%d\n', sum(reaction_data==1), sum(reaction_data==-1), sum(reaction_data==0));

    % CREATE MILP PROBLEM
    milp = model;
    milp.rowlb = model.b;
    milp.rowub = model.b;

    model.exchange_reactions = strmatch('EX_', model.rxns);
    milp.lb(model.exchange_reactions) = -EXCHANGE_FLUX_BOUND;

   milp.lb(milp.lb==-1000) = -inf_const;%-FLUX_BOUND;
   milp.ub(milp.ub==+1000) =inf_const;% FLUX_BOUND;    
    

    % ACTIVE REACTIONS - FORWARD 
    [met_num, rxn_num] = size(model.S);
    v = find(reaction_data==1); n = length(v);
    m1 = sparse( [1:n], v, ones(n,1), n, size(milp.S,2) );
    m2 = eye(n) * (-FLUX_BOUND - ACTIVE_FLUX);
    milp.S = [milp.S, sparse(size(milp.S,1), n) ; m1 m2]; %S is a sparse matrix
    milp.rowlb = [milp.rowlb ; zeros(n,1)-FLUX_BOUND];
    milp.rowub = [milp.rowub ; zeros(n,1)+inf_const];
    milp.var_type = zeros(n,1) + 1;
    milp.var_ind = v;

    % ACTIVE REACTIONS - BACKWARD
    v = find(reaction_data==1 & model.lb<0);  n = length(v);
    m1 = sparse( [1:n], v, ones(n,1), n, size(milp.S,2) );
    m2 = eye(n) * (FLUX_BOUND + ACTIVE_FLUX);
    milp.S = [milp.S, sparse(size(milp.S,1), n) ; m1 m2];
    %milp.rowlb = [milp.rowlb ; zeros(n,1)+FLUX_BOUND];
    %milp.rowub = [milp.rowub ; zeros(n,1)-inf_const];    
    
    milp.rowlb = [milp.rowlb ; zeros(n,1)-inf_const];
%   milp.rowlb = [milp.rowlb ; zeros(n,1)+milp.lb(v)];
    milp.rowub = [milp.rowub ; zeros(n,1)+FLUX_BOUND];

    
    milp.var_type = [milp.var_type; zeros(n,1) - 1];
    milp.var_ind = [milp.var_ind; v];

    % INACTIVE REACTIONS - FORWARD/BACKWARD
    [met_num, rxn_num] = size(model.S);
    v = find(reaction_data==-1); n = length(v);
    m1 = sparse( [1:n], v, ones(n,1), n, size(milp.S,2) );
    m2 = eye(n) * (FLUX_BOUND - INACTIVE_FLUX );

    v2 = find(reaction_data==-1 & model.lb<0);  n2 = length(v2);
    m3 = sparse( [1:n2], v2, ones(n2,1), n2, size(milp.S,2) );
%     m4 = sparse( [1:n2], find(model.lb(v2)<0), 1, n2, n) * (-FLUX_BOUND +
%     INACTIVE_FLUX); This was the problem: V2 instead of V
    m4 = sparse( [1:n2], find(model.lb(v)<0), 1, n2, n) * (-FLUX_BOUND + INACTIVE_FLUX);


    milp.S = [milp.S, sparse(size(milp.S,1), n) ; m1 m2 ; m3 m4];
    milp.rowlb = [milp.rowlb ; zeros(n,1)-inf_const ; zeros(n2,1)-FLUX_BOUND ];
   %milp.rowlb = [milp.rowlb ; zeros(n,1)+milp.lb(v) ; ; zeros(n2,1)-FLUX_BOUND ];
    milp.rowub = [milp.rowub ; zeros(n,1)+FLUX_BOUND ; zeros(n2,1)+inf_const];
    milp.var_type = [milp.var_type; zeros(n,1)];
    milp.var_ind = [milp.var_ind; v];

    % 
    n = size(milp.S,2) - rxn_num;
    milp.c = [zeros(rxn_num,1); ones(n, 1)];
    milp.int_vars = milp.c;
    milp.lb = [milp.lb ; zeros(n, 1)];
    milp.ub = [milp.ub ; zeros(n, 1)+1];
    milp.rxn_num = rxn_num;
    milp.met_num = met_num; 
    milp.gene_num = length(model.genes);
    milp.gene_exp = exp_data;
    milp.reaction_exp = reaction_data;
    milp.model_name = model_name;
    milp.var_num = length(milp.var_type);
end
