#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <time.h>
#ifdef _WIN32
#include <conio.h>
#endif
#include "MetabolicModel.h"
#include "CplexInterface.h"

#define TOL (1e-6)

double GetNum(char *st)
{
	//*st = (unsigned char)toupper((unsigned char)*st);
	double num;
	if (strcmp(st, "-1000") == 0)
		num = -CPX_INFBOUND;
	else if (strcmp(st, "1000") == 0)
		num = CPX_INFBOUND;
	else
		num = atof(st);

	return num;
}

tMetabolicModel::tMetabolicModel(CPXENVptr env)
{
	m_env = env;
	m_lp = NULL;
}

void tMetabolicModel::SetWorkDir1(char *workDir)
{
	strcpy(m_workDir1, workDir);
}
char * tMetabolicModel::AddFullPath1(char *fileName)
{
	strcpy(m_fullPathFileName1, m_workDir1);
	strcat(m_fullPathFileName1, fileName);
	return m_fullPathFileName1;
}

void tMetabolicModel::AppendVector(char *fileName, double *vector, int len, char *delimiter, int is_int, int rewrite_file)
{
	int i;
	FILE *out;

	if (rewrite_file)
		out = fopen(AddFullPath1(fileName), "wt");
	else
		out = fopen(AddFullPath1(fileName), "at");

	if (out == NULL)
	{
		printf("file not found\n");
		exit(0);
	}
	for (i = 0 ; i < len ; i ++)
	{
		if (is_int)
		{
			fprintf(out, "%d%s", (int)(vector[i]>0.5), delimiter);
		}
		else
			fprintf(out, "%e%s", vector[i], delimiter);
	}
	if (strcmp(delimiter, "\n") != 0)
		fprintf(out, "\n");

	fclose(out);
}


// =======================================================================================
// =======================================================================================
// =======================================================================================
// =======================================================================================

void tMetabolicModel::Load(int rows, int cols,
						   char *matrix_file_name,
						   char *constraints_file_name,
						   char *optimization_file_name,
						   char *row_constraints_file_name,
						   char *var_type_file_name,
						   char *result_opt_file_name,
						   char *result_status_file_name,
						   char *result_vector_file_name,
			   			   int calc_result_vector)
{
	m_rows = rows;
	m_cols = cols;
	m_output_result_vector = calc_result_vector;
	//printf("LOAD: m_output_result_vector = %d\n", m_output_result_vector);

	strcpy(m_result_opt_file_name, result_opt_file_name);
	strcpy(m_result_status_file_name, result_status_file_name);
	strcpy(m_result_vector_file_name, result_vector_file_name);

	////////////////////////////////////////////////
	// Stoichiometric matrix
	////////////////////////////////////////////////

	FILE *in = fopen(AddFullPath1(matrix_file_name), "rt");
	if (in == NULL)
	{
		printf("file not found - matrix: %s\n", AddFullPath1(matrix_file_name));
		exit(0);
	}
	char st1[100000];

	int r = 0, c = 0;
	m_v_len = 0;
	int x,y;
	float f;
	int max_col = 0;
	int max_row = 0;
	int i;
	for (i = 0 ; i < m_cols ; i ++)
	{
		m_cn[i] = -1;
		m_cl[i] = 0;
	}

	while (fgets(st1, 100000, in))
	{
		sscanf(st1, "%d %d %f", &y, &x, &f);

		// y is rows, x is cols
		x--;
		y--;

		if (m_cn[x] == -1)
			m_cn[x] = m_v_len;

		m_cl[x] ++;

		m_rn[m_v_len] = y;
		m_v[m_v_len] = f;

		m_v_len ++;

		if (y > m_rows || x > m_cols)
		{
			printf("Error reading matrix file\n");
			exit(0);
		}
		if (y > max_row)
			max_row = y;

		if (x > max_col)
			max_col = x;

	}
	fclose(in);

	////////////////////////////////////////////////
	// Variable Constraints
	////////////////////////////////////////////////

	in = fopen(AddFullPath1(constraints_file_name), "rt");
	if (in == NULL)
	{
		printf("file not found %s\n", AddFullPath1(constraints_file_name));
		exit(0);
	}
	c = 0;
	while (fgets(st1, 100000, in))
	{
		int i = 0;
		char *st = st1;
		while (1)
		{
			char *st_min=strtok(st, " \n");
			if (st_min == NULL)
				break;
			st = NULL;
			char *st_max=strtok(NULL, " \n");

			m_col_lb[c] = GetNum(st_min);
			m_col_ub[c] = GetNum(st_max);
			i ++;
		}
		//constraints_table_size = i;

		c ++;
	}
	fclose(in);

	////////////////////////////////////////////////
	// Optimization
	////////////////////////////////////////////////

	in = fopen(AddFullPath1(optimization_file_name), "rt");
	if (in == NULL)
	{
		printf("file not found - optimization\n");
		exit(0);
	}
	c = 0;

	memset(m_batch_opt_table, 0, sizeof(double)*OPT_BATCH_SIZE*MAX_COL_NUM);
	m_batch_opt_num = 0;
	while (fgets(st1, 100000, in))
	{
		int x,y;
		float f;
		sscanf(st1, "%d %d %f", &y, &x, &f);
		m_batch_opt_table[x-1][y-1] = f;
		if (x > m_batch_opt_num)
			m_batch_opt_num = x;
	}

	fclose(in);


	////////////////////////////////////////////////
	// Row Constraints
	////////////////////////////////////////////////

	// Note that in common cases both lb and ub of row constraints are zero.
	in = fopen(AddFullPath1(row_constraints_file_name), "rt");
	if (in == NULL)
	{
		printf("file not found - row constraints\n");
		exit(0);
	}
	c = 0;
	while (fgets(st1, 100000, in))
	{
		//int i = 0;
		char *st = st1;
		while (1)
		{
			char *st_min=strtok(st, " \n");
			if (st_min == NULL)
				break;
			st = NULL;
			char *st_max=strtok(NULL, " \n");

			m_row_lb[c] = GetNum(st_min);
			m_row_ub[c] = GetNum(st_max);
			//i ++;
		}
		//constraints_table_size = i;
		c ++;
	}

	fclose(in);


	////////////////////////////////////////////////
	// Variable types
	////////////////////////////////////////////////

	in = fopen(AddFullPath1(var_type_file_name), "rt");
	if (in == NULL)
	{
		printf("5.file not found %s\n", AddFullPath1(var_type_file_name));
		exit(0);
	}
	c = 0;
	m_is_milp = 0;
	while (fgets(st1, 1000, in))
	{
		// this value can be '0' (for real values), '1' (Integer), '2' (for Boolean values).
		m_variable_type[c] = atoi(st1);
		if (m_variable_type[c] > 0)
			m_is_milp = 1;

		c ++;
	}
	if (c != m_cols)
	{
		printf("Error in variable type file\n");
		exit(0);
	}

	fclose(in);
}

//load 2 matrices holding a batch of lb and ub vectors for a batch run
void tMetabolicModel::LoadBatchConstraints (char *lbBatch_constraints_file_name,char *ubBatch_constraints_file_name)
{
	LoadConsntraints (lbBatch_constraints_file_name,false);
	LoadConsntraints (ubBatch_constraints_file_name,true);

	return;
}

//load a sparse matrix holding a batch of lb/ub constraints vectors for a batch run.
//mode=0 ->lb, mode=1 ->ub
void tMetabolicModel::LoadConsntraints (char *batch_constraints_file_name, bool mode)
{
	FILE *in = fopen(AddFullPath1(batch_constraints_file_name), "rt");
	if (in == NULL)
	{
		printf("file not found - batch_constraints\n");
		exit(0);
	}

	char st1[100000];

	int batch_size = 0;
	if (mode)
	{
		memset(m_batch_ub_index_last,0, sizeof(int)*CONSTRAINTS_BATCH_SIZE);
	}
	else
	{
		memset(m_batch_lb_index_last,0, sizeof(int)*CONSTRAINTS_BATCH_SIZE);
	}

	while (fgets(st1, 100000, in))
	{
		int x,y;
		float f;
		// x-batch number, y-reaction number, f- reaction bound
		sscanf(st1, "%d %d %f", &y, &x, &f);

		if ( (x-1) > CONSTRAINTS_BATCH_SIZE)
			{
				printf("Error in batch load, batch size in mode=%d\n", mode);
				exit(0);
			}

		if ((y-1) > m_cols)
			{
				printf("Error in batch load, y val=%d exceeds m_cols in mode =%d\n",y,mode);
				exit(0);
			}

		///coloums in the matrix file become rows in the objects' matrix.
		if (mode)//mode=1 -> ub
		{
			m_batch_ub_index[x-1][m_batch_ub_index_last[x-1]] = y-1;
			m_batch_ub[x-1][m_batch_ub_index_last[x-1]] = f;
			m_batch_ub_index_last[x-1] ++;
			if (m_batch_ub_index_last[x-1] > CONSTRAINTS_BATCH_UPDATE_SIZE)
			{
				printf("Error in batch load, exceeds batch_update_size mode=%d\n",mode);
				exit(0);
			}


		}
		else
		{
			m_batch_lb_index[x-1][m_batch_lb_index_last[x-1]] = y-1;
			m_batch_lb[x-1][m_batch_lb_index_last[x-1]] = f;
			m_batch_lb_index_last[x-1] ++;
			if (m_batch_lb_index_last[x-1] > CONSTRAINTS_BATCH_UPDATE_SIZE)
			{
				printf("Error in batch load, exceeds batch_update_size in mode =%d\n",mode);
				exit(0);
			}
		}
		if (x > batch_size)
			batch_size = x;
	}
	fclose(in);

	//update m_batch_size
	if(batch_size > m_batch_size)
		m_batch_size=batch_size;
}

// =======================================================================================
// =======================================================================================
// =======================================================================================
// =======================================================================================

void tMetabolicModel::InitLp()
{
	double obj[MAX_COL_NUM];
	memset(obj, 0, sizeof(double)*MAX_COL_NUM);

	char variable_type[MAX_COL_NUM];
	for (int i = 0 ; i < MAX_COL_NUM ; i ++)
		if (m_variable_type[i] == 0)
			variable_type[i] = 'C';
		else
		if (m_variable_type[i] == 1)
			variable_type[i] = 'I';
		else
			variable_type[i] = 'B';

	CreateCplexProblem(m_env, m_lp,
					   m_cols, m_rows, obj,
					   m_cn, m_cl, m_rn, m_v, m_col_lb, m_col_ub,
					   m_row_lb, m_row_ub,
					   variable_type);
}

double tMetabolicModel::ComputeLP(int output_result_vector)
{
	double opt_val;
	if (output_result_vector)
 		opt_val = SolveCplexProblem(m_env, m_lp, m_opt_vector, &m_opt_status);
	else
		opt_val = SolveCplexProblem(m_env, m_lp, NULL, &m_opt_status);

	return opt_val;
}

double tMetabolicModel::ComputeMIP(int output_result_vector)
{
	double opt_val;
	if (output_result_vector)
 		opt_val = SolveCplexMipProblem(m_env, m_lp, m_opt_vector, &m_opt_status,m_cols);
	else
		opt_val = SolveCplexMipProblem(m_env, m_lp, NULL, &m_opt_status,m_cols);

	if (m_opt_status == 101 || m_opt_status == 102)
		m_opt_status = 1;
	else
	if (m_opt_status == 0)
		m_opt_status = 0;

	return opt_val;
}

double tMetabolicModel::ComputeOpt(int output_result_vector)
{
	double opt_val;
	if (m_is_milp==0)
	{
		//printf ("\nLP Problem\n");
		opt_val = ComputeLP(output_result_vector);
	}
	else
	{
		//printf ("\nMILP Problem\n");
		opt_val = ComputeMIP(output_result_vector);
	}

	return opt_val;
}

void tMetabolicModel::OutputVector(int rewrite_file)
{
	if (m_output_result_vector == 1)
	{
		AppendVector(m_result_vector_file_name, (double *)m_opt_vector, m_cols, " ", 0, rewrite_file);
	}
	else
	{
		if (m_output_result_vector == 2)
		{
			double values[MAX_COL_NUM];
			int c = 0;
			for (int i = 0 ; i < m_cols ; i ++)
			{
				//printf("var_type = %d, val = %e\n", m_variable_type[i], m_opt_vector[i]);
				if (m_variable_type[i] != 0)
					values[c++] = m_opt_vector[i];
			}

			AppendVector(m_result_vector_file_name, (double *)values, c, " ", 1, rewrite_file);
		}
	}
}

void tMetabolicModel::RunBatchOpt()
{
	double result_opt[OPT_BATCH_SIZE];
	double result_status[OPT_BATCH_SIZE];

	for (int i = 0 ; i < m_batch_opt_num ; i ++)
	{
		// UPDATE OBJECTIVE
		int indices[MAX_COL_NUM];
		double values[MAX_COL_NUM];
		for (int c = 0 ; c < m_cols ; c++)
		{
			indices[c] = c;
			values[c] = m_batch_opt_table[i][c];
		}
		UpdateCplexProblemObj(m_env, m_lp, m_cols, indices, values);

		// RUN OPTIMIZATION
		double opt_val = ComputeOpt(m_output_result_vector);

		result_opt[i] = opt_val;
		result_status[i] = m_opt_status;

		// SAVE RESULT VECTOR
		OutputVector(i==0);
	}

	AppendVector(m_result_opt_file_name, (double *)result_opt, m_batch_opt_num, "\n", 0, 1);
	AppendVector(m_result_status_file_name, (double *)result_status, m_batch_opt_num, "\n", 0, 1);
}


void tMetabolicModel::BatchSkipModeUpdate(int batch_start, int batch_num, double *result_opt, double *result_status, double wt_opt)
{
	for (int x = batch_start-1 ; x < batch_start+batch_num-1 ; x++)
	{
		int i;
		int found = 1;
		for (i = 0 ; i < m_batch_ub_index_last[x] ; i++)
		{
			int r = m_batch_ub_index[x][i];
			double ub = m_col_ub[r] + m_batch_ub[x][i];
			if (m_opt_vector[r] > ub+TOL)
				found = 0;
		}

		for (i = 0 ; i < m_batch_lb_index_last[x] ; i++)
		{
			int r = m_batch_lb_index[x][i];
			double lb = m_col_lb[r] + m_batch_lb[x][i];
			if (m_opt_vector[r] < lb-TOL)
				found = 0;
		}

		if (found)
		{
			result_opt[x] = wt_opt;
			result_status[x] = 1;
			///printf("\tFound Skip: %d\n", x-batch_start+2);
		}
	}
}

void tMetabolicModel::RunBatchConstraints(int batch_start, int batch_num, int batch_skip_mode)
{
	printf("START: BATCH %d\n", batch_start); fflush(stdout);
	double result_opt[CONSTRAINTS_BATCH_SIZE];
	double result_status[CONSTRAINTS_BATCH_SIZE];
	for (int i = 0 ; i < CONSTRAINTS_BATCH_SIZE ; i ++)
	{
		result_opt[i] = -1;
		result_status[i] = -1;
	}

	// UPDATE OBJECTIVE
	int indices[MAX_COL_NUM];
	double values[MAX_COL_NUM];
	for (int c = 0 ; c < m_cols ; c++)
	{
		indices[c] = c;
		values[c] = m_batch_opt_table[0][c];
	}
	UpdateCplexProblemObj(m_env, m_lp, m_cols, indices, values);

	double wt_opt;
	if (batch_skip_mode)
	{
		printf("Batch Skip Mode..\n");
		wt_opt = ComputeOpt(1);
		BatchSkipModeUpdate(batch_start, batch_num, result_opt, result_status, wt_opt);
		//for (int i = 0 ; i < m_cols ; i ++)
			//printf("%d: %f\n", i+1, m_opt_vector[i]);

	}

	for (int x = batch_start-1 ; x < batch_start+batch_num-1 ; x++)
	{
		printf("MILP %d/%d:\n", x-batch_start+2, batch_num); fflush(stdout);
		if (result_status[x] != -1)
		{
			printf("\tSkipping..\n"); fflush(stdout);
			continue;
		}

		// UPDATE CONSTRAINTS
		double lb_values[MAX_COL_NUM];
		double ub_values[MAX_COL_NUM];
		memcpy(lb_values,  m_col_lb, sizeof(double)*MAX_COL_NUM);
		memcpy(ub_values,  m_col_ub, sizeof(double)*MAX_COL_NUM);

		int i;
		for (i = 0 ; i < m_batch_ub_index_last[x] ; i++)
			ub_values[m_batch_ub_index[x][i]] += m_batch_ub[x][i];

		for (i = 0 ; i < m_batch_lb_index_last[x] ; i++)
			lb_values[m_batch_lb_index[x][i]] += m_batch_lb[x][i];

		UpdateCplexProblemBounds(m_env,m_lp, m_cols, indices, lb_values, ub_values)	;

		// RUN OPTIMIZATION
		double opt_val = ComputeOpt(m_output_result_vector);

		result_opt[x] = opt_val;
		result_status[x] = m_opt_status;

		if (batch_skip_mode && opt_val==wt_opt)
		{
			BatchSkipModeUpdate(batch_start, batch_num, result_opt, result_status, wt_opt);
		}

		// SAVE RESULT VECTOR
		OutputVector(x==0);
	}

	//write status and opt values to files.
	AppendVector(m_result_opt_file_name, (double *)&result_opt[batch_start-1], batch_num, "\n", 0, 1);
	AppendVector(m_result_status_file_name, (double *)&result_status[batch_start-1], batch_num, "\n", 0, 1);
	printf("END: BATCH %d\n", batch_start); fflush(stdout);
}
