#ifndef _METABOLIC_MODEL_H_
#define _METABOLIC_MODEL_H_


#include "cplex.h"

#define MAX_COL_NUM						10000
#define MAX_ROW_NUM						10000

#define OPT_BATCH_SIZE					2*1100
#define CONSTRAINTS_BATCH_SIZE			35000
#define CONSTRAINTS_BATCH_UPDATE_SIZE	100
#define CONSTRAINTS_BATCH_CORRECTION	1234

//using namespace std;


class tMetabolicModel
{
public:
	tMetabolicModel(CPXENVptr env);
	void Load(int rows, int cols,
			  char *matrix_file_name,
			  char *constraints_file_name,
			  char *optimization_file_name,
			  char *row_constraints_file_name,
			  char *var_type_file_name,
			  char *result_opt_file_name,
			  char *result_status_file_name,
			  char *result_vector_file_name,
			  int calc_result_vector);

	void InitLp();

	void RunBatchOpt();
	void RunBatchConstraints(int batch_start, int batch_num, int batch_skip_mode);

	void LoadBatchConstraints (char *lbBatch_constraints_file_name,char *ubBatch_constraints_file_name);
	void SetWorkDir1(char *workDir);

private:
	void LoadConsntraints (char *batch_constraints_file_name,bool mode);

	double ComputeLP(int output_result_vector);
	double ComputeMIP(int output_result_vector);
	double ComputeOpt(int output_result_vector);
	void BatchSkipModeUpdate(int batch_start, int batch_num, double *result_opt, double *result_status, double wt_opt);

	char *AddFullPath1(char *fileName);
	void AppendVector(char *fileName, double *vector, int len, char *delimiter, int is_int, int rewrite_file);
	void OutputVector(int rewrite_file);


private:
	char m_fullPathFileName1[1024];
	char m_workDir1[1024];
	char m_result_opt_file_name[200];
	char m_result_status_file_name[200];
	char m_result_vector_file_name[200];
	int m_output_result_vector;

	double	m_opt_vector[MAX_COL_NUM+1];
	int		m_opt_status;

	// CPLEX DATA
	CPXENVptr m_env;
	CPXLPptr m_lp;

	int m_cn[MAX_COL_NUM+1];
	int m_cl[MAX_COL_NUM+1];
	int m_rn[200000];
	double m_v[200000];
	int m_v_len;

	int		m_rows, m_cols;
	int		m_is_milp;
	double m_col_lb[MAX_COL_NUM], m_col_ub[MAX_COL_NUM];
	double m_row_lb[MAX_ROW_NUM], m_row_ub[MAX_ROW_NUM];
	int m_variable_type[MAX_COL_NUM];

	// OPTIMIZATION BATCH
	int m_batch_opt_num;
	double m_batch_opt_table[OPT_BATCH_SIZE][MAX_COL_NUM];

	// CONSTRAINTS BATCH
	int		m_batch_size;

	int		m_batch_lb_index[CONSTRAINTS_BATCH_SIZE][CONSTRAINTS_BATCH_UPDATE_SIZE];
	int		m_batch_lb_index_last[CONSTRAINTS_BATCH_SIZE];
	double	m_batch_lb[CONSTRAINTS_BATCH_SIZE][CONSTRAINTS_BATCH_UPDATE_SIZE];

	int		m_batch_ub_index[CONSTRAINTS_BATCH_SIZE][CONSTRAINTS_BATCH_UPDATE_SIZE];
	int		m_batch_ub_index_last[CONSTRAINTS_BATCH_SIZE];
	double	m_batch_ub[CONSTRAINTS_BATCH_SIZE][CONSTRAINTS_BATCH_UPDATE_SIZE];
};
#endif


