#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#ifdef _WIN32
#include <conio.h>
#endif
#include "MetabolicModel.h"
#include "CplexInterface.h"

//PLS NOTE: BATCH_CORRECTION IS A VALUE THAT SHOULD BE ADDED TO EACH CHANGED LB/UB VALUE IN A BATCH RUN
//LB/UB THAT ARE SET TO +/- 1000 WILL BE DEFINED AS CPXunbounded!!!

int main(int argc, char *argv[])
{
	tMetabolicModel * metabolicModel = NULL;
	CPXENVptr env = NULL;

	srand(time(NULL));

	if (argc < 3)
	{
		printf("\nError in parameters\n");
		exit(0);
	}

	char *work_dir = argv[1];
	printf ("work_dir: %s\n",work_dir);
	int prob_num = atoi(argv[2]); 
	int rows = atoi(argv[3]);
	int cols = atoi(argv[4]);
	int output_result_vector = atoi(argv[5]);
	int run_batch_constraints = atoi(argv[6]);
	int batch_start = 1;
	int batch_num = 0;
	int batch_skip_mode = 0;
	if (run_batch_constraints)
	{
		batch_start = atoi(argv[7]);
		batch_num = atoi(argv[8]);
		batch_skip_mode = atoi(argv[9]);
	}
	if (batch_skip_mode) 
		output_result_vector = 1;

	printf("PARAMS: output_result_vector = %d\n", output_result_vector);


	char matrix_file_name[256];
	sprintf(matrix_file_name, "prob_%d_mat.txt", prob_num);

	char row_constrains_file_name[256];
	sprintf(row_constrains_file_name, "prob_%d_row_bound.txt", prob_num);

	char col_constraints_file_name[256];
	sprintf(col_constraints_file_name, "prob_%d_col_bound.txt", prob_num);

	char optimization_file_name[256];
	sprintf(optimization_file_name, "prob_%d_opt.txt", prob_num);

	char var_type_file_name[256];
	sprintf(var_type_file_name, "prob_%d_int.txt", prob_num);

	char lb_batch_constraints_file_name[256];
	sprintf(lb_batch_constraints_file_name, "prob_%d_batch_lb.txt", prob_num);

	char ub_batch_constraints_file_name[256];
	sprintf(ub_batch_constraints_file_name, "prob_%d_batch_ub.txt", prob_num);

	char result_opt_file_name[256];
	sprintf(result_opt_file_name, "prob_%d_result_%d.txt", prob_num, batch_start);

	char result_status_file_name[256];
	sprintf(result_status_file_name, "prob_%d_status_%d.txt", prob_num, batch_start);

	char result_vector_file_name[256];
	sprintf(result_vector_file_name, "prob_%d_vector_%d.txt", prob_num, batch_start);


	/* Initialize the CPLEX environment */
	int status = openEnv(env);
	if (status == ERROR_NO_AVAIABLE_LICENSE)
		return -1;

	metabolicModel = new tMetabolicModel(env);
	metabolicModel->SetWorkDir1(work_dir);

	metabolicModel->Load(rows, cols, matrix_file_name, col_constraints_file_name,
						 optimization_file_name, row_constrains_file_name,
						 var_type_file_name,
						 result_opt_file_name,
						 result_status_file_name,
						 result_vector_file_name,
						 output_result_vector);

	metabolicModel->InitLp();

	if (run_batch_constraints == 1)
	{
		metabolicModel->LoadBatchConstraints(lb_batch_constraints_file_name, ub_batch_constraints_file_name);
		metabolicModel->RunBatchConstraints(batch_start, batch_num, batch_skip_mode);
	}
	else
	{
		metabolicModel->RunBatchOpt();
	}


	return 0;
}
